from papercandy.train import *
from papercandy.config import *
from papercandy.drawing import *
from papercandy.network import *
from papercandy.dataloader import *


CONFIG = new_config("conf.pcc")  # default path
